(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
    typeof define === 'function' && define.amd ? define(['exports'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.Pdfsvex = {}));
}(this, (function (exports) { 'use strict';

    function noop() { }
    function assign(tar, src) {
        // @ts-ignore
        for (const k in src)
            tar[k] = src[k];
        return tar;
    }
    function run(fn) {
        return fn();
    }
    function blank_object() {
        return Object.create(null);
    }
    function run_all(fns) {
        fns.forEach(run);
    }
    function is_function(thing) {
        return typeof thing === 'function';
    }
    function safe_not_equal(a, b) {
        return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
    }
    function is_empty(obj) {
        return Object.keys(obj).length === 0;
    }
    function create_slot(definition, ctx, $$scope, fn) {
        if (definition) {
            const slot_ctx = get_slot_context(definition, ctx, $$scope, fn);
            return definition[0](slot_ctx);
        }
    }
    function get_slot_context(definition, ctx, $$scope, fn) {
        return definition[1] && fn
            ? assign($$scope.ctx.slice(), definition[1](fn(ctx)))
            : $$scope.ctx;
    }
    function get_slot_changes(definition, $$scope, dirty, fn) {
        if (definition[2] && fn) {
            const lets = definition[2](fn(dirty));
            if ($$scope.dirty === undefined) {
                return lets;
            }
            if (typeof lets === 'object') {
                const merged = [];
                const len = Math.max($$scope.dirty.length, lets.length);
                for (let i = 0; i < len; i += 1) {
                    merged[i] = $$scope.dirty[i] | lets[i];
                }
                return merged;
            }
            return $$scope.dirty | lets;
        }
        return $$scope.dirty;
    }
    function update_slot(slot, slot_definition, ctx, $$scope, dirty, get_slot_changes_fn, get_slot_context_fn) {
        const slot_changes = get_slot_changes(slot_definition, $$scope, dirty, get_slot_changes_fn);
        if (slot_changes) {
            const slot_context = get_slot_context(slot_definition, ctx, $$scope, get_slot_context_fn);
            slot.p(slot_context, slot_changes);
        }
    }

    function append(target, node) {
        target.appendChild(node);
    }
    function insert(target, node, anchor) {
        target.insertBefore(node, anchor || null);
    }
    function detach(node) {
        node.parentNode.removeChild(node);
    }
    function element(name) {
        return document.createElement(name);
    }
    function text(data) {
        return document.createTextNode(data);
    }
    function space() {
        return text(' ');
    }
    function attr(node, attribute, value) {
        if (value == null)
            node.removeAttribute(attribute);
        else if (node.getAttribute(attribute) !== value)
            node.setAttribute(attribute, value);
    }
    function set_attributes(node, attributes) {
        // @ts-ignore
        const descriptors = Object.getOwnPropertyDescriptors(node.__proto__);
        for (const key in attributes) {
            if (attributes[key] == null) {
                node.removeAttribute(key);
            }
            else if (key === 'style') {
                node.style.cssText = attributes[key];
            }
            else if (key === '__value') {
                node.value = node[key] = attributes[key];
            }
            else if (descriptors[key] && descriptors[key].set) {
                node[key] = attributes[key];
            }
            else {
                attr(node, key, attributes[key]);
            }
        }
    }
    function children(element) {
        return Array.from(element.childNodes);
    }
    function toggle_class(element, name, toggle) {
        element.classList[toggle ? 'add' : 'remove'](name);
    }

    let current_component;
    function set_current_component(component) {
        current_component = component;
    }
    function get_current_component() {
        if (!current_component)
            throw new Error('Function called outside component initialization');
        return current_component;
    }
    function onMount(fn) {
        get_current_component().$$.on_mount.push(fn);
    }

    const dirty_components = [];
    const binding_callbacks = [];
    const render_callbacks = [];
    const flush_callbacks = [];
    const resolved_promise = Promise.resolve();
    let update_scheduled = false;
    function schedule_update() {
        if (!update_scheduled) {
            update_scheduled = true;
            resolved_promise.then(flush);
        }
    }
    function add_render_callback(fn) {
        render_callbacks.push(fn);
    }
    let flushing = false;
    const seen_callbacks = new Set();
    function flush() {
        if (flushing)
            return;
        flushing = true;
        do {
            // first, call beforeUpdate functions
            // and update components
            for (let i = 0; i < dirty_components.length; i += 1) {
                const component = dirty_components[i];
                set_current_component(component);
                update(component.$$);
            }
            set_current_component(null);
            dirty_components.length = 0;
            while (binding_callbacks.length)
                binding_callbacks.pop()();
            // then, once components are updated, call
            // afterUpdate functions. This may cause
            // subsequent updates...
            for (let i = 0; i < render_callbacks.length; i += 1) {
                const callback = render_callbacks[i];
                if (!seen_callbacks.has(callback)) {
                    // ...so guard against infinite loops
                    seen_callbacks.add(callback);
                    callback();
                }
            }
            render_callbacks.length = 0;
        } while (dirty_components.length);
        while (flush_callbacks.length) {
            flush_callbacks.pop()();
        }
        update_scheduled = false;
        flushing = false;
        seen_callbacks.clear();
    }
    function update($$) {
        if ($$.fragment !== null) {
            $$.update();
            run_all($$.before_update);
            const dirty = $$.dirty;
            $$.dirty = [-1];
            $$.fragment && $$.fragment.p($$.ctx, dirty);
            $$.after_update.forEach(add_render_callback);
        }
    }
    const outroing = new Set();
    let outros;
    function transition_in(block, local) {
        if (block && block.i) {
            outroing.delete(block);
            block.i(local);
        }
    }
    function transition_out(block, local, detach, callback) {
        if (block && block.o) {
            if (outroing.has(block))
                return;
            outroing.add(block);
            outros.c.push(() => {
                outroing.delete(block);
                if (callback) {
                    if (detach)
                        block.d(1);
                    callback();
                }
            });
            block.o(local);
        }
    }

    function get_spread_update(levels, updates) {
        const update = {};
        const to_null_out = {};
        const accounted_for = { $$scope: 1 };
        let i = levels.length;
        while (i--) {
            const o = levels[i];
            const n = updates[i];
            if (n) {
                for (const key in o) {
                    if (!(key in n))
                        to_null_out[key] = 1;
                }
                for (const key in n) {
                    if (!accounted_for[key]) {
                        update[key] = n[key];
                        accounted_for[key] = 1;
                    }
                }
                levels[i] = n;
            }
            else {
                for (const key in o) {
                    accounted_for[key] = 1;
                }
            }
        }
        for (const key in to_null_out) {
            if (!(key in update))
                update[key] = undefined;
        }
        return update;
    }
    function mount_component(component, target, anchor, customElement) {
        const { fragment, on_mount, on_destroy, after_update } = component.$$;
        fragment && fragment.m(target, anchor);
        if (!customElement) {
            // onMount happens before the initial afterUpdate
            add_render_callback(() => {
                const new_on_destroy = on_mount.map(run).filter(is_function);
                if (on_destroy) {
                    on_destroy.push(...new_on_destroy);
                }
                else {
                    // Edge case - component was destroyed immediately,
                    // most likely as a result of a binding initialising
                    run_all(new_on_destroy);
                }
                component.$$.on_mount = [];
            });
        }
        after_update.forEach(add_render_callback);
    }
    function destroy_component(component, detaching) {
        const $$ = component.$$;
        if ($$.fragment !== null) {
            run_all($$.on_destroy);
            $$.fragment && $$.fragment.d(detaching);
            // TODO null out other refs, including component.$$ (but need to
            // preserve final state?)
            $$.on_destroy = $$.fragment = null;
            $$.ctx = [];
        }
    }
    function make_dirty(component, i) {
        if (component.$$.dirty[0] === -1) {
            dirty_components.push(component);
            schedule_update();
            component.$$.dirty.fill(0);
        }
        component.$$.dirty[(i / 31) | 0] |= (1 << (i % 31));
    }
    function init(component, options, instance, create_fragment, not_equal, props, dirty = [-1]) {
        const parent_component = current_component;
        set_current_component(component);
        const $$ = component.$$ = {
            fragment: null,
            ctx: null,
            // state
            props,
            update: noop,
            not_equal,
            bound: blank_object(),
            // lifecycle
            on_mount: [],
            on_destroy: [],
            on_disconnect: [],
            before_update: [],
            after_update: [],
            context: new Map(parent_component ? parent_component.$$.context : options.context || []),
            // everything else
            callbacks: blank_object(),
            dirty,
            skip_bound: false
        };
        let ready = false;
        $$.ctx = instance
            ? instance(component, options.props || {}, (i, ret, ...rest) => {
                const value = rest.length ? rest[0] : ret;
                if ($$.ctx && not_equal($$.ctx[i], $$.ctx[i] = value)) {
                    if (!$$.skip_bound && $$.bound[i])
                        $$.bound[i](value);
                    if (ready)
                        make_dirty(component, i);
                }
                return ret;
            })
            : [];
        $$.update();
        ready = true;
        run_all($$.before_update);
        // `false` as a special case of no DOM component
        $$.fragment = create_fragment ? create_fragment($$.ctx) : false;
        if (options.target) {
            if (options.hydrate) {
                const nodes = children(options.target);
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.l(nodes);
                nodes.forEach(detach);
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.c();
            }
            if (options.intro)
                transition_in(component.$$.fragment);
            mount_component(component, options.target, options.anchor, options.customElement);
            flush();
        }
        set_current_component(parent_component);
    }
    /**
     * Base class for Svelte components. Used when dev=false.
     */
    class SvelteComponent {
        $destroy() {
            destroy_component(this, 1);
            this.$destroy = noop;
        }
        $on(type, callback) {
            const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
            callbacks.push(callback);
            return () => {
                const index = callbacks.indexOf(callback);
                if (index !== -1)
                    callbacks.splice(index, 1);
            };
        }
        $set($$props) {
            if (this.$$set && !is_empty($$props)) {
                this.$$.skip_bound = true;
                this.$$set($$props);
                this.$$.skip_bound = false;
            }
        }
    }

    const ATTRIBUTE_NAME_DATA_CREATED_FROM_UID = 'data-created-from-uid';
    const ATTRIBUTE_NAME_DYNAMIC_COMPONENT_ID = 'data-component-id';
    const ATTRIBUTE_NAME_PAGE_NUMBER = 'data-page-number';
    const CLASS_NAME_DYNAMIC_COMPONENT = 'dynamic-component';
    const ATTRIBUTE_NAME_SAVED_ID = 'saved-id';
    const ATTRIBUTE_NAME_ID = 'id';
    const CLASS_NAME_PAGE = 'sheet';
    const CLASS_NAME_CONTAINS_SPLITTET_TEXT_ACROSS_PAGES =
        'contains-splittet-text-across-pages';

    /* src/components/DynamicComponent.svelte generated by Svelte v3.38.2 */

    function create_fragment$1(ctx) {
    	let span;
    	let span_levels = [{ class: CLASS_NAME_DYNAMIC_COMPONENT }, /*attributes*/ ctx[0]];
    	let span_data = {};

    	for (let i = 0; i < span_levels.length; i += 1) {
    		span_data = assign(span_data, span_levels[i]);
    	}

    	return {
    		c() {
    			span = element("span");
    			set_attributes(span, span_data);
    		},
    		m(target, anchor) {
    			insert(target, span, anchor);
    		},
    		p(ctx, [dirty]) {
    			set_attributes(span, span_data = get_spread_update(span_levels, [
    				{ class: CLASS_NAME_DYNAMIC_COMPONENT },
    				dirty & /*attributes*/ 1 && /*attributes*/ ctx[0]
    			]));
    		},
    		i: noop,
    		o: noop,
    		d(detaching) {
    			if (detaching) detach(span);
    		}
    	};
    }

    function instance$1($$self, $$props, $$invalidate) {
    	let { componentId } = $$props;
    	let { attributes = {} } = $$props;
    	attributes[ATTRIBUTE_NAME_DYNAMIC_COMPONENT_ID] = componentId;

    	$$self.$$set = $$props => {
    		if ("componentId" in $$props) $$invalidate(1, componentId = $$props.componentId);
    		if ("attributes" in $$props) $$invalidate(0, attributes = $$props.attributes);
    	};

    	return [attributes, componentId];
    }

    class DynamicComponent extends SvelteComponent {
    	constructor(options) {
    		super();
    		init(this, options, instance$1, create_fragment$1, safe_not_equal, { componentId: 1, attributes: 0 });
    	}
    }

    class Page {
        constructor(options, index) {
            this.options = options;
            this.index = index;
            this.domAnchor = this.generate();
        }

        generate() {
            const page = document.createElement('div');
            page.classList.add('sheet');
            page.setAttribute('data-page-number', this.index);

            this.layoutComponent = new this.options.layout({
                target: page,
                props: {
                    pageIndex: this.index,
                    content: undefined,
                },
            });

            return page;
        }

        get dom() {
            return this.domAnchor;
        }
        get contentDom() {
            return this.domAnchor.querySelector('.page-content');
        }

        hasOverflow() {
            return (
                this.domAnchor.scrollHeight > this.domAnchor.clientHeight
                // this.domAnchor.scrollWidth > this.domAnchor.clientWidth
            );
        }

        save() {
            this.layoutComponent.$set({ content: this.domAnchor });
        }

        getNodeIfExists(dataUid) {
            return this.contentDom.querySelector(
                `[${ATTRIBUTE_NAME_DATA_CREATED_FROM_UID}="${dataUid}"]`
            );
        }
    }

    var IDX=256, HEX=[], SIZE=256, BUFFER;
    while (IDX--) HEX[IDX] = (IDX + 256).toString(16).substring(1);

    function uid(len) {
    	var i=0, tmp=(len || 11);
    	if (!BUFFER || ((IDX + tmp) > SIZE*2)) {
    		for (BUFFER='',IDX=0; i < SIZE; i++) {
    			BUFFER += HEX[Math.random() * 256 | 0];
    		}
    	}

    	return BUFFER.substring(IDX, IDX++ + tmp);
    }

    class BaseNode {
        /**
         * Checks if a node is a node
         * @param {HTMLElement} node node to determine the type of
         * @returns {boolean} true if the node is of specified type
         */
        static is(node) {
            return true;
        }

        /**
         * Creates a new node based on another node
         * @param {HTMLElement} node node to recreate
         * @returns {HTMLElement} created node with the copied content
         */
        static createFrom(node) {
            let createdNode = undefined;

            if (node.tagName === 'svg' || node instanceof SVGElement) {
                // svg nodes and all its children need to be created differently: https://www.brightec.co.uk/blog/svg-wouldnt-render
                // how to detect svg elements: https://stackoverflow.com/questions/20748836/how-do-i-tell-if-a-dom-element-is-html-or-svg
                createdNode = document.createElementNS(
                    'http://www.w3.org/2000/svg',
                    node.tagName
                );
            } else {
                createdNode = document.createElement(node.tagName);
            }

            // copy all attributes to the new created node
            // these attributes can contain classnames or style informations
            if (node.attributes) {
                const attributes = Array.prototype.slice.call(node.attributes);
                let attribute;
                while ((attribute = attributes.pop())) {
                    createdNode.setAttribute(
                        attribute.nodeName,
                        attribute.nodeValue
                    );
                }

                if (createdNode.hasAttribute(ATTRIBUTE_NAME_SAVED_ID)) {
                    BaseNode.restoreId(createdNode);
                }
            }
            return createdNode;
        }

        /**
         * Restores the id from a saved id in order to avoid duplicated ids in the document tree
         * @param {HTMLElement} node to restore the id from
         */
        static restoreId(node) {
            const idValue = node.getAttribute(ATTRIBUTE_NAME_SAVED_ID);
            node.removeAttribute(ATTRIBUTE_NAME_SAVED_ID);
            node.setAttribute(ATTRIBUTE_NAME_ID, idValue);
        }

        /**
         * checks if the node has the page-break-inside property set to avoid
         * @param {HTMLElement} node node to check
         * @returns {boolean} result of the check
         */
        static hasPageBreakInsideAvoid(node) {
            const computedStyles = window.getComputedStyle(node);
            return computedStyles.getPropertyValue('page-break-inside') === 'avoid';
        }

        /**
         * checks if the node has the page-break-after property set to always
         * @param {HTMLElement} node node to check
         * @returns {boolean} result of the check
         */
        static hasPageBreakAfterAlways(node) {
            const computedStyles = window.getComputedStyle(node);
            return computedStyles.getPropertyValue('page-break-after') === 'always';
        }
    }

    class TextNode {
        /**
         * Checks if a node is a text node
         * @param {HTMLElement} node node to determine the type of
         * @returns {boolean} true if the node is of specified type
         */
        static is(node) {
            return node.nodeType === Node.TEXT_NODE;
        }

        /**
         * Creates a new text node based on another node
         * @param {HTMLElement} node node to recreate
         * @returns {HTMLElement} created node with the copied content
         */
        static createFrom(node) {
            return document.createTextNode(node.textContent);
        }
    }

    class CommentNode {
        /**
         * Checks if a node is a comment node
         * @param {HTMLElement} node node to determine the type of
         * @returns {boolean} true if the node is of specified type
         */
        static is(node) {
            return node.nodeType === Node.COMMENT_NODE;
        }

        /**
         * Creates a new comment node based on another node
         * @param {HTMLElement} node node to recreate
         * @returns {HTMLElement} created node with the copied content
         */
        static createFrom(node) {
            return document.createComment(node.data);
        }
    }

    class TextSplitter {
        /**
         * Splits a text node into two pages. We try to fit as much content as possible
         * into the current page, and only create a new page if the current page has overflown.
         * @param {HTMLElement} nodeToCopy the node that should be recreated
         * @param {HTMLElement} node the node created from `nodeToCopy`, that will be appended to the next page
         * @param {Page} currentPage the page we are currently creating
         * @param {HTMLElement} appendToNode the node that all children should append to
         */
        static splitToPages(nodeToCopy, node, currentPage, appendToNode) {
            // split the contents of the words at whitespace
            const words = nodeToCopy.textContent
                .replaceAll('\n\t', ' ')
                .replaceAll('\n', ' ')
                .replaceAll('\t', ' ')
                .split(' ');

            // get or create the parent
            const parentNodeCurrentPage = TextSplitter.determineParent(
                currentPage,
                appendToNode,
                nodeToCopy
            );

            // create the text node for the current page
            const textNodeCurrentPage = TextNode.createFrom(nodeToCopy);
            // append it to its parent
            parentNodeCurrentPage.appendChild(textNodeCurrentPage);
            // and make sure to clear its content
            textNodeCurrentPage.textContent = '';

            // add a class to indicate that some text from this node was split to the next one
            parentNodeCurrentPage.classList.add(
                CLASS_NAME_CONTAINS_SPLITTET_TEXT_ACROSS_PAGES
            );

            // now add as much words as possible to the current page
            const remainingWords = TextSplitter.addWordsUntilOverflow(
                words,
                textNodeCurrentPage,
                currentPage
            );

            // and add the remaining words to the next page
            node.textContent = remainingWords;
        }

        /**
         * Gets or creates the parent of the node
         * @param {Page} currentPage the page we are currently creating
         * @param {HTMLElement} appendToNode the node that all children should append to
         * @param {HTMLElement} nodeToCopy the node that should be recreated
         */
        static determineParent(currentPage, appendToNode, nodeToCopy) {
            // as we are starting to do some dom manipulation here, it might happen that
            // the expected parent is higher in the dom then we are.

            // so get the uid of the original parent
            const parentUid = nodeToCopy.parentElement.getAttribute(
                ATTRIBUTE_NAME_DATA_CREATED_FROM_UID
            );
            // and search it in the current page
            let parentNodeCurrentPage = currentPage.getNodeIfExists(parentUid);

            if (!parentNodeCurrentPage) {
                // if the node does not exist, create it
                parentNodeCurrentPage = BaseNode.createFrom(
                    nodeToCopy.parentElement
                );
                // and append it to the page
                appendToNode.appendChild(parentNodeCurrentPage);
            }

            return parentNodeCurrentPage;
        }

        /**
         * Adds words to the current page, as long as it does not overflow
         * @param {string[]} words an array of words that were present in the text-node
         * @param {HTMLElement} textNodeCurrentPage the text-node of the current page, where we can append some text to
         * @param {Page} currentPage the current page we are currently working on
         * @returns {string} an string of words that needs to be appended to the next page, as the current one would overflow
         */
        static addWordsUntilOverflow(words, textNodeCurrentPage, currentPage) {
            let lastSuccessfullWord = -1;

            // iterate over each word
            for (let i = 0; i < words.length; i++) {
                const word = words[i];

                // and append it to the node
                textNodeCurrentPage.textContent += word + ' ';

                if (currentPage.hasOverflow()) {
                    // if the page did overflow, this word is too much
                    // so find the last occurrence of the current word in the node
                    const lastWordIndex =
                        textNodeCurrentPage.textContent.lastIndexOf(word);
                    // and remove it
                    textNodeCurrentPage.textContent =
                        textNodeCurrentPage.textContent.substring(0, lastWordIndex);
                    break;
                } else {
                    // if we successfully added to page, save the index of this word for later reuse
                    lastSuccessfullWord = i;
                }
            }

            // now return the remaining words that could not be appended to the current page, and join them with a whitespace
            return words.slice(lastSuccessfullWord + 1, words.length).join(' ');
        }
    }

    class DynamicComponents {
        /**
         * Instantiates all dynamic components
         * @param {object} options
         * @param {Page[]} pages
         * @param {HTMLElement} generated
         */
        static create(options, pages, generated) {
            // iterate all pages
            pages.forEach((page) => {
                // get all dynamic components by the specified class name
                const dynamicComponentWrappers = page.contentDom.querySelectorAll(
                    '.' + CLASS_NAME_DYNAMIC_COMPONENT
                );
                // iterate over all found dynamic components
                dynamicComponentWrappers.forEach((componentWrapper) => {
                    // get the id of the dynamic component
                    const componentId = componentWrapper.getAttribute(
                        ATTRIBUTE_NAME_DYNAMIC_COMPONENT_ID
                    );

                    // and search it in the config
                    const componentInfo =
                        DynamicComponents.getComponentInfoFromOptions(
                            options,
                            componentId
                        );

                    if (componentInfo != undefined) {
                        // if the components exists in the config, then create it wit all according props
                        new componentInfo.component({
                            target: componentWrapper,
                            props: {
                                ...componentInfo.props,
                                pages: pages,
                                allContent: generated,
                                element: componentWrapper,
                            },
                        });
                    }
                });
            });
        }

        /**
         * Searches a component in the options of pdfsvex
         * @param {object} options
         * @param {string} componentId
         * @returns the component config
         */
        static getComponentInfoFromOptions(options, componentId) {
            if (options.dynamicComponents) {
                return options.dynamicComponents.find((c) => c.id === componentId);
            }
            return undefined;
        }
    }

    class Pager {
        /**
         * Creates a new pager
         * @param {HTMLElement} content content that should be paged
         * @param {HTMLElement} generated node to append the generated content to
         * @param {*} options options
         */
        constructor(content, generated, options) {
            this.generated = generated;
            this.options = options;
            this.content = content;
        }

        pageNow() {
            this.removeIds(this.content);

            this.wayToRoot = [];
            this.pages = [];

            this.generated.innerHTML = '';

            this.currentPage = this.createAndAddPage();
            const appendToNode = this.currentPage.contentDom;

            this.walk(appendToNode, this.content, true);

            this.finalize();
        }

        /**
         * Removes all id attributes from all child elements to avoid having duplicated ids in document tree
         * @param {HTMLElement} node root node, remove all ids from all child elements
         */
        removeIds(node) {
            // get all elements with ids
            const nodeList = node.querySelectorAll(`[${ATTRIBUTE_NAME_ID}]`);

            // and replace the id attribute with a custom one
            nodeList.forEach((n) => {
                const idValue = n.getAttribute(ATTRIBUTE_NAME_ID);
                n.removeAttribute(ATTRIBUTE_NAME_ID);
                n.setAttribute(ATTRIBUTE_NAME_SAVED_ID, idValue);
            });
        }

        /**
         * Travels the dom and initiates all accurate actions for each of the nodes
         * Can be seen as the brain of this program
         * @param {HTMLElement} appendToNode node to append new children's to
         * @param {HTMLElement} nodeToCopy the node that should be recreated
         * @param {boolean} root determines if we are the root node
         * @returns {HTMLElement} the node the parent should append its remaining nodes to
         */
        walk(appendToNode, nodeToCopy, root = false) {
            if (!TextNode.is(nodeToCopy) && !CommentNode.is(nodeToCopy)) {
                // if this is a real html node
                // add a unique id to each of the original html nodes
                nodeToCopy.setAttribute(
                    ATTRIBUTE_NAME_DATA_CREATED_FROM_UID,
                    uid()
                );
            }

            let node = undefined;
            if (root) {
                // if we are the root, just append to this node
                node = appendToNode;
            } else {
                // if we are a child of the root node
                // create the child with all appropriate attributes
                node = this.createNode(nodeToCopy);
                // and append it to the dom
                appendToNode.appendChild(node);
            }

            let avoidBreakInside = false;
            let pageBreakAfter = false;
            // check if we need to pay attention on some css attributes
            if (!TextNode.is(nodeToCopy) && !CommentNode.is(nodeToCopy)) {
                avoidBreakInside = BaseNode.hasPageBreakInsideAvoid(node);
                pageBreakAfter = BaseNode.hasPageBreakAfterAlways(node);
            }

            // check if the page is currently overflowing
            let hasOverflow = this.currentPage.hasOverflow();

            let skipChildren = false;
            if (avoidBreakInside) {
                // if the current element should not be breaked inside
                // iterate over all of its children and create the appropriate nodes recursively
                nodeToCopy.childNodes.forEach((child) => {
                    this.createNodesRecursive(node, child);
                });

                // since we just handled all of the children, and will add them to the next page
                // we don't need to re-add them later on
                skipChildren = true;

                // redetermin if the current page has overflown
                hasOverflow = this.currentPage.hasOverflow();
            }

            if (
                hasOverflow ||
                pageBreakAfter ||
                (hasOverflow && avoidBreakInside)
            ) {
                // if the page is overflowing,
                // or we need to do a page break
                // or we should avoid breaking inside of this element
                // proceed

                // as the page has overflown or should be added to the next page
                // remove the node from the current page
                appendToNode.removeChild(node);

                if (
                    TextNode.is(nodeToCopy) &&
                    appendToNode.parentNode &&
                    appendToNode.children.length == 0
                ) {
                    // if we are a text-node and we were just removed
                    // from the current page as the page has overflown,
                    // it might happen that our parent is in example a p-tag
                    // with no children.

                    // if this is the case, also remove the parent
                    const parent = appendToNode.parentElement;
                    parent.removeChild(appendToNode);

                    // and reassign the node future children should append to
                    appendToNode = parent;
                }

                if (TextNode.is(nodeToCopy)) {
                    // if we are a text node, it might be possible to break the text
                    // into two parts. One part for the current page, and one for the next one.
                    // Thats why we try to split the text node into two parts here
                    TextSplitter.splitToPages(
                        nodeToCopy,
                        node,
                        this.currentPage,
                        appendToNode
                    );
                }

                // if we arrived here, we know we will need another page
                // so create it
                const nextPage = this.createAndAddPage();
                // and save that all future html elements should be added to the new page
                appendToNode = nextPage.contentDom;
                this.currentPage = nextPage;

                // so now lets assume that the last node we worked on was some highly nested node
                // and we decided to create a new page. So appending the new node to the root of the
                // new page, might completely break the layout. Thats why we need to recreate the exact same
                // structure as we had before creating a new page.
                // Thats why we save each parent we traverse into the `wayToRoot` which describes the way from the
                // page root down to last node we worked on
                this.wayToRoot.forEach((wtr) => {
                    // create the node
                    const wayToRootNode = BaseNode.createFrom(wtr);
                    // add node to new parent
                    appendToNode.appendChild(wayToRootNode);
                    // make the current the node every child should append to
                    appendToNode = wayToRootNode;
                });

                // append the actual node to the new page
                appendToNode.appendChild(node);
            }

            // always save the way to the page-root into the `wayToRoot` for later reuse
            if (!root) this.wayToRoot.push(nodeToCopy);

            if (!skipChildren) {
                // iterate all child nodes and recursively call this function
                appendToNode = node;
                nodeToCopy.childNodes.forEach((child) => {
                    appendToNode = this.walk(appendToNode, child);
                });
            }

            // remove the last element from `wayToRoot` as we have successfully processed it now
            if (!root) this.wayToRoot.pop();

            if (avoidBreakInside) {
                // if this is a node with avoid break inside, it is enough to return the current node
                return appendToNode;
            }
            // as we are now going one node up in the dom-tree, return the parent of the current node
            return appendToNode.parentElement;
        }

        /**
         * Create a new page and append it to the dom
         * (optionally, save the last page)
         * @returns {Page} the next page
         */
        createAndAddPage() {
            if (this.currentPage) {
                this.currentPage.save();
            }

            const page = new Page(this.options, this.pages.length + 1);
            this.pages.push(page);

            this.generated.appendChild(page.dom);
            return page;
        }

        /**
         * Creates a new html node
         * @param {HTMLElement} nodeToCopy a node that should be recreated
         * @returns {HTMLElement} the newly created node
         */
        createNode(nodeToCopy) {
            let node = undefined;
            if (TextNode.is(nodeToCopy)) {
                // text
                node = TextNode.createFrom(nodeToCopy);
            } else if (CommentNode.is(nodeToCopy)) {
                // comment
                node = CommentNode.createFrom(nodeToCopy);
            } else {
                // all other nodes
                node = BaseNode.createFrom(nodeToCopy);
            }

            return node;
        }

        /**
         * Recursively creates the new node
         * @param {HTMLElement} appendTo the node the children should be appended to
         * @param {HTMLElement} node the node to append
         */
        createNodesRecursive(appendTo, node) {
            const newAppendTo = this.createNode(node);
            appendTo.appendChild(newAppendTo);

            node.childNodes.forEach((child) => {
                this.createNodesRecursive(newAppendTo, child);
            });
        }

        finalize() {
            DynamicComponents.create(this.options, this.pages, this.generated);
        }
    }

    /* src/components/PdfDocument.svelte generated by Svelte v3.38.2 */

    function add_css() {
    	var style = element("style");
    	style.id = "svelte-1rco1pq-style";
    	style.textContent = ".content.svelte-1rco1pq{display:none}@media screen{.wrapper2.svelte-1rco1pq{display:table;margin:0 auto}.wrapper.svelte-1rco1pq{display:block}.generated.svelte-1rco1pq{display:inline-block}}@page{margin:0}body{margin:0}.sheet{margin:0;overflow:hidden;position:relative;box-sizing:border-box;page-break-after:always}.generated.A3 .sheet{width:297mm;height:419mm}.generated.A3.landscape .sheet{width:420mm;height:296mm}.generated.A4 .sheet{width:210mm;height:296mm}.generated.A4.landscape .sheet{width:297mm;height:209mm}.generated.A5 .sheet{width:148mm;height:209mm}.generated.A5.landscape .sheet{width:210mm;height:147mm}.generated.letter .sheet{width:216mm;height:279mm}.generated.letter.landscape .sheet{width:280mm;height:215mm}.generated.legal .sheet{width:216mm;height:356mm}.generated.legal.landscape .sheet{width:357mm;height:215mm}@media screen{body{background:#e0e0e0}.sheet{background:white;box-shadow:0 0.5mm 2mm rgba(0, 0, 0, 0.3);margin:5mm}}@media print{.generated.A3.landscape{width:420mm}.generated.A3,.generated.A4.landscape{width:297mm}.generated.A4,.generated.A5.landscape{width:210mm}.generated.A5{width:148mm}.generated.letter,.generated.legal{width:216mm}.generated.letter.landscape{width:280mm}.generated.legal.landscape{width:357mm}}";
    	append(document.head, style);
    }

    // (34:10) This is a slot component, you need to provide the content
    function fallback_block(ctx) {
    	let t;

    	return {
    		c() {
    			t = text("This is a slot component, you need to provide the content");
    		},
    		m(target, anchor) {
    			insert(target, t, anchor);
    		},
    		d(detaching) {
    			if (detaching) detach(t);
    		}
    	};
    }

    function create_fragment(ctx) {
    	let div0;
    	let t0;
    	let div3;
    	let div2;
    	let div1;
    	let t1;
    	let div1_class_value;
    	let current;
    	const default_slot_template = /*#slots*/ ctx[9].default;
    	const default_slot = create_slot(default_slot_template, ctx, /*$$scope*/ ctx[8], null);
    	const default_slot_or_fallback = default_slot || fallback_block();

    	return {
    		c() {
    			div0 = element("div");
    			if (default_slot_or_fallback) default_slot_or_fallback.c();
    			t0 = space();
    			div3 = element("div");
    			div2 = element("div");
    			div1 = element("div");
    			t1 = text("Loading...");
    			attr(div0, "class", "content svelte-1rco1pq");
    			attr(div1, "class", div1_class_value = "generated " + /*style*/ ctx[0] + " svelte-1rco1pq");
    			toggle_class(div1, "landscape", /*landscape*/ ctx[1]);
    			attr(div2, "class", "wrapper svelte-1rco1pq");
    			attr(div3, "class", "wrapper2 svelte-1rco1pq");
    		},
    		m(target, anchor) {
    			insert(target, div0, anchor);

    			if (default_slot_or_fallback) {
    				default_slot_or_fallback.m(div0, null);
    			}

    			/*div0_binding*/ ctx[10](div0);
    			insert(target, t0, anchor);
    			insert(target, div3, anchor);
    			append(div3, div2);
    			append(div2, div1);
    			append(div1, t1);
    			/*div1_binding*/ ctx[11](div1);
    			/*div2_binding*/ ctx[12](div2);
    			current = true;
    		},
    		p(ctx, [dirty]) {
    			if (default_slot) {
    				if (default_slot.p && (!current || dirty & /*$$scope*/ 256)) {
    					update_slot(default_slot, default_slot_template, ctx, /*$$scope*/ ctx[8], dirty, null, null);
    				}
    			}

    			if (!current || dirty & /*style*/ 1 && div1_class_value !== (div1_class_value = "generated " + /*style*/ ctx[0] + " svelte-1rco1pq")) {
    				attr(div1, "class", div1_class_value);
    			}

    			if (dirty & /*style, landscape*/ 3) {
    				toggle_class(div1, "landscape", /*landscape*/ ctx[1]);
    			}
    		},
    		i(local) {
    			if (current) return;
    			transition_in(default_slot_or_fallback, local);
    			current = true;
    		},
    		o(local) {
    			transition_out(default_slot_or_fallback, local);
    			current = false;
    		},
    		d(detaching) {
    			if (detaching) detach(div0);
    			if (default_slot_or_fallback) default_slot_or_fallback.d(detaching);
    			/*div0_binding*/ ctx[10](null);
    			if (detaching) detach(t0);
    			if (detaching) detach(div3);
    			/*div1_binding*/ ctx[11](null);
    			/*div2_binding*/ ctx[12](null);
    		}
    	};
    }

    function instance($$self, $$props, $$invalidate) {
    	let { $$slots: slots = {}, $$scope } = $$props;
    	let { options } = $$props;
    	let { style = "A4" } = $$props;
    	let { landscape = false } = $$props;
    	let { present = false } = $$props;
    	let content;
    	let generated;
    	let wrapper;
    	let pager;

    	onMount(() => {
    		pager = new Pager(content, generated, options);
    	});

    	function pageNow() {
    		pager.pageNow();

    		if (present) {
    			// scale on page to the screen height
    			const windowHeight = window.innerHeight;

    			const pageHeight = generated.querySelector(".sheet").clientHeight;
    			const zoom = windowHeight / pageHeight;
    			$$invalidate(3, generated.style.transform = "scale(" + zoom + ", " + zoom + ")", generated);
    			$$invalidate(3, generated.style.transformOrigin = "top", generated);
    		}
    	}

    	function div0_binding($$value) {
    		binding_callbacks[$$value ? "unshift" : "push"](() => {
    			content = $$value;
    			$$invalidate(2, content);
    		});
    	}

    	function div1_binding($$value) {
    		binding_callbacks[$$value ? "unshift" : "push"](() => {
    			generated = $$value;
    			$$invalidate(3, generated);
    		});
    	}

    	function div2_binding($$value) {
    		binding_callbacks[$$value ? "unshift" : "push"](() => {
    			wrapper = $$value;
    			$$invalidate(4, wrapper);
    		});
    	}

    	$$self.$$set = $$props => {
    		if ("options" in $$props) $$invalidate(5, options = $$props.options);
    		if ("style" in $$props) $$invalidate(0, style = $$props.style);
    		if ("landscape" in $$props) $$invalidate(1, landscape = $$props.landscape);
    		if ("present" in $$props) $$invalidate(6, present = $$props.present);
    		if ("$$scope" in $$props) $$invalidate(8, $$scope = $$props.$$scope);
    	};

    	return [
    		style,
    		landscape,
    		content,
    		generated,
    		wrapper,
    		options,
    		present,
    		pageNow,
    		$$scope,
    		slots,
    		div0_binding,
    		div1_binding,
    		div2_binding
    	];
    }

    class PdfDocument extends SvelteComponent {
    	constructor(options) {
    		super();
    		if (!document.getElementById("svelte-1rco1pq-style")) add_css();

    		init(this, options, instance, create_fragment, safe_not_equal, {
    			options: 5,
    			style: 0,
    			landscape: 1,
    			present: 6,
    			pageNow: 7
    		});
    	}

    	get pageNow() {
    		return this.$$.ctx[7];
    	}
    }

    exports.ATTRIBUTE_NAME_DATA_CREATED_FROM_UID = ATTRIBUTE_NAME_DATA_CREATED_FROM_UID;
    exports.ATTRIBUTE_NAME_DYNAMIC_COMPONENT_ID = ATTRIBUTE_NAME_DYNAMIC_COMPONENT_ID;
    exports.ATTRIBUTE_NAME_ID = ATTRIBUTE_NAME_ID;
    exports.ATTRIBUTE_NAME_PAGE_NUMBER = ATTRIBUTE_NAME_PAGE_NUMBER;
    exports.ATTRIBUTE_NAME_SAVED_ID = ATTRIBUTE_NAME_SAVED_ID;
    exports.CLASS_NAME_CONTAINS_SPLITTET_TEXT_ACROSS_PAGES = CLASS_NAME_CONTAINS_SPLITTET_TEXT_ACROSS_PAGES;
    exports.CLASS_NAME_DYNAMIC_COMPONENT = CLASS_NAME_DYNAMIC_COMPONENT;
    exports.CLASS_NAME_PAGE = CLASS_NAME_PAGE;
    exports.DynamicComponent = DynamicComponent;
    exports.PdfDocument = PdfDocument;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
